package com.ruoyi.business.service.impl;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.atomic.AtomicReference;
import java.util.stream.Collectors;

import com.ruoyi.business.domain.HwDeviceModeFunction;
import com.ruoyi.business.domain.VO.DeviceModeVo;
import com.ruoyi.business.domain.VO.HwDeviceVo;
import com.ruoyi.business.mapper.HwDeviceModeFunctionMapper;
import com.ruoyi.common.core.constant.HwDictConstants;
import com.ruoyi.common.core.constant.TdEngineConstants;
import com.ruoyi.common.core.enums.DataTypeEnums;
import com.ruoyi.common.core.utils.DateUtils;
import com.ruoyi.common.core.utils.NumberUtils;
import com.ruoyi.tdengine.api.RemoteTdEngineService;
import com.ruoyi.tdengine.api.domain.TdHistorySelectDto;
import com.ruoyi.tdengine.api.domain.TdSelectDto;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.ruoyi.business.mapper.HwDeviceMapper;
import com.ruoyi.business.domain.HwDevice;
import com.ruoyi.business.service.IHwDeviceService;

/**
 * 设备信息Service业务层处理
 *
 * @author xins
 * @date 2023-08-24
 */
@Service
public class HwDeviceServiceImpl implements IHwDeviceService {
    @Autowired
    private HwDeviceMapper hwDeviceMapper;
    @Autowired
    private HwDeviceModeFunctionMapper hwDevieModeFunctionMapper;
    @Autowired
    private RemoteTdEngineService remoteTdEgineService;

    /**
     * 查询设备信息
     *
     * @param deviceId 设备信息主键
     * @return 设备信息
     */
    @Override
    public HwDevice selectHwDeviceByDeviceId(Long deviceId) {
        return hwDeviceMapper.selectHwDeviceByDeviceId(deviceId);
    }

    /**
     * 查询设备信息,返回监控单元名称
     *
     * @param deviceId 设备信息主键
     * @return 设备信息
     */

    @Override
    public HwDevice selectHwDeviceJoinByDeviceId(Long deviceId) {
        return hwDeviceMapper.selectHwDeviceJoinByDeviceId(deviceId);
    }
    /**
     * 查询设备信息列表
     *
     * @param hwDevice 设备信息
     * @return 设备信息
     */
    @Override
    public List<HwDevice> selectHwDeviceList(HwDevice hwDevice) {
        return hwDeviceMapper.selectHwDeviceList(hwDevice);
    }

    /**
     * 查询关联设备信息列表
     *
     * @param deviceId 设备信息主键
     * @return 设备信息
     */
    @Override
    public List<HwDevice> selectLinkDeviceList(Long deviceId) {
        return hwDeviceMapper.selectLinkedDevices(deviceId);
    }

    /**
     * 新增设备信息
     *
     * @param hwDevice 设备信息
     * @return 结果
     */
    @Override
    public int insertHwDevice(HwDevice hwDevice) {
        hwDevice.setCreateTime(DateUtils.getNowDate());
        return hwDeviceMapper.insertHwDevice(hwDevice);
    }

    /**
     * 查询3.	设备各类型占所有设备数量百分比饼状图（左下角）
     */
    @Override
    public List<DeviceModeVo> selectDeviceModeNameVo() {
        List<DeviceModeVo> list = hwDeviceMapper.selectDeviceModeNameVo();
        Integer value = 0;
        if (list.size() > 10) {
            for (int i = 9; i < list.size(); i++) {
                if (list.get(i).sum == null) {
                    list.get(i).sum = 0;
                }
                value = value + list.get(i).sum;
            }
        }
//        DeviceModeVo hwMonitorUnitTypeVo = new DeviceModeVo();
//        hwMonitorUnitTypeVo.setDeviceModeName("其他");
//        hwMonitorUnitTypeVo.setSum(value);
        Integer sums = 0;
        for (int i = 0; i < list.size(); i++) {
            if (list.get(i).sum == null) {
                list.get(i).sum = 0;
            }
            sums = sums + list.get(i).sum;
        }
        for (int i = 0; i < list.size(); i++) {
            list.get(i).setPercentage(NumberUtils.getPercentage(list.get(i).getSum(), sums));
            System.out.println(list.get(i).toString());
        }


        return list;
//        if (list.size() > 10) {
//            list.add(10, hwMonitorUnitTypeVo);
//            return list.subList(0, 10);
//        } else {
//            return list;
//        }
    }


    /**
     * 修改设备信息
     *
     * @param hwDevice 设备信息
     * @return 结果
     */
    @Override
    public int updateHwDevice(HwDevice hwDevice) {
        hwDevice.setUpdateTime(DateUtils.getNowDate());
        return hwDeviceMapper.updateHwDevice(hwDevice);
    }

    /**
     * 批量删除设备信息
     *
     * @param deviceIds 需要删除的设备信息主键
     * @return 结果
     */
    @Override
    public int deleteHwDeviceByDeviceIds(Long[] deviceIds) {
        return hwDeviceMapper.deleteHwDeviceByDeviceIds(deviceIds);
    }

    /**
     * 删除设备信息信息
     *
     * @param deviceId 设备信息主键
     * @return 结果
     */
    @Override
    public int deleteHwDeviceByDeviceId(Long deviceId) {
        return hwDeviceMapper.deleteHwDeviceByDeviceId(deviceId);
    }


    /**
     * @return Map<String,List<HwDeviceVo>>
     * @description 设备监测页面，根据监控单元获取设备监控信息
     * @author xins
     * @date 2023-09-15 16:59
     */
    @Override
    public Map<String,List<HwDeviceVo>> getDevicesByMonitor(HwDevice queryHwDevice) {
        Map<String,List<HwDeviceVo>> devicesMap = new HashMap<String,List<HwDeviceVo>>();
        List<HwDeviceVo> controlDeviceVos = new ArrayList<HwDeviceVo>();
        List<HwDeviceVo> acquisitionDeviceVos = new ArrayList<HwDeviceVo>();
        Long sceneId = queryHwDevice.getSceneId();
        String databaseName = TdEngineConstants.getDatabaseName(sceneId);

        List<HwDevice> hwDevices = hwDeviceMapper.selectHwDeviceList(queryHwDevice);
        hwDevices.forEach(hwDevice -> {

            Long deviceId = hwDevice.getDeviceId();
            String tableName = TdEngineConstants.getDeviceDataTableName(deviceId);
            TdSelectDto tdSelectDto = new TdSelectDto();
            tdSelectDto.setDatabaseName(databaseName);
            tdSelectDto.setTableName(tableName);
            List<Map<String, Object>> deviceLatestDataMapList = (List<Map<String, Object>>) this.remoteTdEgineService.getLatestData(tdSelectDto).getData();

            HwDeviceModeFunction queryDeviceModeFunction = new HwDeviceModeFunction();
            queryDeviceModeFunction.setDeviceModeId(hwDevice.getDeviceModeId());
            List<HwDeviceModeFunction> deviceModeFunctions = hwDevieModeFunctionMapper.selectHwDeviceModeFunctionList(queryDeviceModeFunction);
            List<HwDeviceModeFunction> dds = deviceModeFunctions.stream()
                    .filter(dmf -> dmf.getFunctionMode().equals(HwDictConstants.FUNCTION_MODE_ATTRIBUTE)
                            && dmf.getDataType() != null && String.valueOf(dmf.getDataType()).equals(String.valueOf(DataTypeEnums.TINYINT.getDataCode()))).collect(Collectors.toList());

            HwDeviceVo hwDeviceVo = new HwDeviceVo();
            hwDeviceVo.setDeviceName(hwDevice.getDeviceName());
            if (dds != null && dds.size() > 0) {
                hwDeviceVo.setDeviceType(HwDictConstants.CONTROL_DEVICE);
                HwDeviceModeFunction dd = dds.get(0);
                List<Map<String, Object>> ddValueMapList = new ArrayList<Map<String, Object>>();
                Map<String, Object> ddValueMap = new HashMap<String, Object>();

                deviceLatestDataMapList.forEach(deviceLatestDataMap -> {
                    ddValueMap.put(dd.getFunctionIdentifier(), deviceLatestDataMap.get(dd.getFunctionIdentifier()) == null
                            ? 0 : deviceLatestDataMap.get(dd.getFunctionIdentifier()));
                });
                ddValueMapList.add(ddValueMap);

                hwDeviceVo.setDeviceDataMaps(ddValueMapList);
                controlDeviceVos.add(hwDeviceVo);
            } else {
                hwDeviceVo.setDeviceType(HwDictConstants.ACQUISITION_DEVICE);
                hwDeviceVo.setDeviceDataMaps(deviceLatestDataMapList);
                acquisitionDeviceVos.add(hwDeviceVo);
            }

        });

        devicesMap.put(HwDictConstants.CONTROL_DEVICE_NAME, controlDeviceVos);
        devicesMap.put(HwDictConstants.ACQUISITION_DEVICE_NAME, acquisitionDeviceVos);

        return devicesMap;
    }

}
